@@ -39,7 +39,7 @@ mod extern_rust_method_swift_class_placement {
 public class SomeType: SomeTypeRefMut {
     var isOwned: Bool = true
 
-    override init(ptr: UnsafeMutableRawPointer) {
+    public override init(ptr: UnsafeMutableRawPointer) {
         super.init(ptr: ptr)
     }
 
@@ -50,41 +50,41 @@ public class SomeType: SomeTypeRefMut {
     }
 }
 extension SomeType {
-    func a() {
+    public func a() {
         __swift_bridge__$SomeType$a({isOwned = false; return ptr;}())
     }
 
-    func b() {
+    public func b() {
         __swift_bridge__$SomeType$b({isOwned = false; return ptr;}())
     }
 }
 public class SomeTypeRefMut: SomeTypeRef {
-    override init(ptr: UnsafeMutableRawPointer) {
+    public override init(ptr: UnsafeMutableRawPointer) {
         super.init(ptr: ptr)
     }
 }
 extension SomeTypeRefMut {
-    func e() {
+    public func e() {
         __swift_bridge__$SomeType$e(ptr)
     }
 
-    func f() {
+    public func f() {
         __swift_bridge__$SomeType$f(ptr)
     }
 }
 public class SomeTypeRef {
     var ptr: UnsafeMutableRawPointer
 
-    init(ptr: UnsafeMutableRawPointer) {
+    public init(ptr: UnsafeMutableRawPointer) {
         self.ptr = ptr
     }
 }
 extension SomeTypeRef {
-    func c() {
+    public func c() {
         __swift_bridge__$SomeType$c(ptr)
     }
 
-    func d() {
+    public func d() {
         __swift_bridge__$SomeType$d(ptr)
     }
 }
